/*
 schupdpp.p - Schema Upate Procedure Processing Subroutine

 This procedure runs the pre-update and/or post-update processing
 procedures specified in the corresponding control file.
*/
DEFINE INPUT  PARAMETER plPreUpdate AS LOGICAL NO-UNDO.

DEFINE VARIABLE cCtrlFile  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cDir       AS CHARACTER NO-UNDO.
DEFINE VARIABLE cFilename  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cProcDir   AS CHARACTER NO-UNDO.
DEFINE VARIABLE i          AS INTEGER   NO-UNDO.

DEFINE TEMP-TABLE ttProcCtrl
  FIELD Completed     AS CHARACTER
  FIELD DatabaseName  AS CHARACTER
  FIELD Procpath      AS CHARACTER
  FIELD DataDirectory AS CHARACTER.

DEFINE SHARED STREAM Supdlog.

DEFINE STREAM sCtrl.

/*
 Set the appropriate directory location for the control file
 and the pre/post processing procedures.
*/
cProcDir = (IF plPreUpdate THEN "rtb_idat/pre/" ELSE "rtb_idat/post/").

/*
 Determine if there is a control file for this stage.
*/
cCtrlFile = SEARCH(cProcDir + "procctrl").
  
IF cCtrlFile = ? THEN
  RETURN.

PUT STREAM Supdlog UNFORMATTED
  SUBSTITUTE("Performing &1-Update Processing...", IF plPreUpdate THEN "Pre" ELSE "Post") SKIP.
  
/*
 Load the control file contents.
*/
INPUT STREAM sCtrl FROM VALUE(cCtrlFile) NO-ECHO.

REPEAT TRANSACTION:
  CREATE ttProcCtrl.  
  IMPORT STREAM sCtrl ttProcCtrl.
END.

INPUT STREAM sCtrl CLOSE.

/*
 Process the entries from the control file...
*/
$PROC_LOOP:
FOR EACH ttProcCtrl BREAK BY ttProcCtrl.DatabaseName ON ERROR UNDO, LEAVE:

  IF ttProcCtrl.DatabaseName = "" THEN NEXT.

  IF FIRST-OF(ttProcCtrl.DatabaseName) THEN DO:
    DELETE ALIAS DICTDB.
    CREATE ALIAS DICTDB FOR DATABASE VALUE(ttProcCtrl.DatabaseName).
  END.

  /* If Completed is "Y" then the procedure has already run */
  IF ttProcCtrl.Completed = "Y" THEN
    NEXT.

  cFilename = SEARCH(cProcDir + ttProcCtrl.Procpath).

  IF ttProcCtrl.DataDirectory = "" THEN
    ttProcCtrl.DataDirectory =  "rtb_idat/tempdata/pre". 
  
  ASSIGN
    ttProcCtrl.DataDirectory = REPLACE(ttProcCtrl.DataDirectory,"~\","/")
    cDir                     = "".

  /* Create data directory path as necessary */
  DO i = 1 TO NUM-ENTRIES(ttProcCtrl.DataDirectory, "/"):

    cDir = cDir + (IF cDir = "" THEN "" ELSE "/") + ENTRY(i,ttProcCtrl.DataDirectory, "/").

    IF SEARCH(cDir) = ? THEN
      OS-CREATE-DIR VALUE(cDir).
  END.

  DO ON ERROR UNDO $PROC_LOOP, LEAVE $PROC_LOOP:

    PUT STREAM Supdlog UNFORMATTED "Running " cFilename "..." SKIP.

    RUN VALUE(cFilename) (INPUT ttProcCtrl.DataDirectory).

    ttProcCtrl.Completed = "Y".

  END.

END. /* $proc_loop */

/*
 Write updated records to the control file.
*/
OUTPUT STREAM sCtrl TO VALUE(cCtrlFile).

FOR EACH ttProcCtrl:
  EXPORT STREAM sCtrl ttProcCtrl.
  DELETE ttProcCtrl.
END.

OUTPUT STREAM sCtrl CLOSE.
 
